
//--------------------------------------------------------
// sketch: Lighting.pde - by Gerd Platl
//         handle lighting parameters of a 3d scene
// date:   v1.0  2012-06-07
//         v1.1  2013-03-23
//--------------------------------------------------------
// material syntax: 
//     ambient(color)          // RGB/HSB
//     emissive(color)         // RGB/HSB 
//     shininess(float shine)  // degree of shininess >= 0
//     specular(color)         // RGB/HSB
// light syntax:
//     lights()                       // set default light values
//     noLights()                     // disable lightning
//     lightFalloff(const,lin,quad)   // default (1, 0, 0)
//     lightSpecular(color)           // default (0, 0, 0)
//     ambientLight(color)            // light from all sides
//     directionalLight(color,dir)    // default (128,128,128, 0,0,-1)
//     pointLight(color,pos)          // to add a point light
//     spotLight(color,pos,dir,angle) // to add a spot light
//--------------------------------------------------------

// hue values constants 0..360°
int hueRed = 0;
int hueOrange = 30; 
int hueYellow = 60;
int hueYellowGreen = 90;
int hueGreen = 120;  
int hueGreenBlue = 150;
int hueCyan = 180;
int hueCyanBlue = 210;
int hueBlue = 240;
int hueViolet = 270;
int hueMagenta = 300;

int iAmbient = 100;      color cAmbient = color(255, 255, 222);
int iAmbientHue = 50;    color cAmbientLight = color(222, 222, 188);

int iSpecularHue = 40;  
int iSpecularSat = 100;    color cSpecular = color(1, 1, 1);
int iSpecular = 255;
float fShininess = 15.0;

int iEmissiveHue = 0;  
int iEmissiveSat = 0;    color cEmissive = color(0, 0, 0);

int iDirectHue = 200;    
int iDirectSat = 0;      color cDirectional = color(1, 1, 1);

int iPointHue = 60;     
int iPointSat = 100;     color cPoint = color(1, 1, 1);

int iSpotHue = 40;     
int iSpotSat = 255;        color cSpot = color(1, 1, 1);

float specularX=0, specularY=400, specularZ=-400;  // specularLight direction
float directX=400, directY=-400, directZ=-200;  // directionalLight direction
float pointX=160, pointY=-20, pointZ=140;   // pointlight position
float spotPosX=200, spotPosY=20, spotPosZ=240;   // spotlight position
float spotDirX=0, spotDirY=0, spotDirZ= -100;   // spotlight direction

//--------------------------------------------------------
void randomLights()
{
  println ("randomLights:");

  iAmbient = int(random(256));
  iAmbientHue = int(random(360));    

  iSpecularHue = int(random(360));  
//  iSpecularSat = 20;   
  iSpecular = int(random(256));
//  fShininess = 1.0;

  iEmissiveHue = int(random(360));
//  iEmissiveSat = int(random(256));

  iDirectHue = int(random(360));
//  iDirectSat = int(random(256));

  iPointHue = int(random(360));
//  iPointSat = int(random(256));

  iSpotHue = int(random(360));
//  iSpotSat = int(random(256));

  initLights();
}
//--------------------------------------------------------
void initLights()
{
  // println ("initLights:");

  colorMode(HSB, 360,255,255);
  cAmbientLight = color (iAmbientHue, 255, iAmbient);
  cSpecular     = color (iSpecularHue, iSpecularSat, iSpecularSat);
  cEmissive     = color (iEmissiveHue, iEmissiveSat, iEmissiveSat);
  cDirectional  = color (iDirectHue, iDirectSat, iDirectSat);
  cPoint        = color (iPointHue, iPointSat, iPointSat);
  cSpot         = color (iSpotHue, iSpotSat, iSpotSat);

  colorMode(RGB, 255);
  cAmbient      = color (iAmbient, iAmbient, iAmbient);
}
//--------------------------------------------------------
// set all kind of lights
//--------------------------------------------------------
void setLights()
{
  //println ("setLights:");
  
  noLights();

  // pointLight
  directionalLight(red(cDirectional), green(cDirectional), blue(cDirectional), // Color
                   directX, directY, directZ); // lightning direction

  // pointLight
  if (iPointSat > 0)
  {
    pointY = 120*sin(frameCount*0.04);
    pointLight(red(cPoint), green(cPoint), blue(cPoint),  // Color
               pointX, pointY, pointZ);  // Position
  }

  // spotLight
  if (iSpotSat > 0)
  {
    spotPosX = 40+100*sin(frameCount*0.03);
    spotLight(red(cSpot), green(cSpot), blue(cSpot),  // Color
              spotPosX, spotPosY, spotPosZ,  // Position
              spotDirX, spotDirY, spotDirZ,  // Direction
              PI, 40);      // Angle= 90°, Concentration
  }
  
  // ambientLight
  ambient(cAmbient);
  ambientLight(red(cAmbientLight), green(cAmbientLight), blue(cAmbientLight));

  // specularLight
  lightSpecular(red(cSpecular), green(cSpecular), blue(cSpecular));    // color
  directionalLight(red(cSpecular), green(cSpecular), blue(cSpecular),  // color
                   specularX, specularY, specularZ);     // lightning direction
  specular(iSpecular, iSpecular, iSpecular);
  shininess(fShininess);

  // emissive light
  emissive(cEmissive);
}

