
//--------------------------------------------------------
// sketch: PG_Lighting_Test.pde - by Gerd Platl
//         Interactive testframe for light experiments.
//         Primary goal is to test different light values
//         in different combinations for using in other sketches.
// tested: with processing v1.5.1
//         with processing v2.0a6 .. v2.0a8
// date:   v1.0  2012-04-23  inital release
//         v1.1  2012-06-07  extracted code to 'Lighting.pde'
//                           corrected for processing v2.0
//         v1.2  2013-03-31  dynamic resolution
//         v1.3  2017-01-23  now working with JavaScript
//
// tags:   light, pointLight, spotLight, lighting, 3d, animation, 
//         sphere, testing
// input:  change slider values or
//         rotate test shape up/down
// keys:   blanc  toggle rotation
//           a  toggle axis
//           f  toggle framesPerSecond
//           w  toggle resolution
//           n  set random light values
//           r  reset all light values
//           i  turn off light intensities
//           s  save as picture 'LightningTest.png'
//          +/- zoom in/out
//--------------------------------------------------------

String title = ">>> Lighting Test v1.3 <<<";

boolean debugMode = false;
boolean isPaused = false;
boolean doRotation = true;
boolean showFPS = false;
boolean showResolution = false;
boolean doSmoothing = true;
boolean drawAxis = false;

color textColor = color(255);
float viewDistance = -40.0;
float rotx = -0.12;         // current rotation about x-axis
float spin = 0.0;           // current object rotation

HScrollbar[] scrollbar = new HScrollbar[14];

Button  ResetButton, RandomButton, OffButton, SaveButton;

static int 
  xs =  490,  // x position slider
  xt = xs+8,  // x position text
  bh =   60,  // button height
  yh =   31,  // y height slider
  sh = yh-2,  // slider height
  sw =  304,  // slider width
  th = sh-5;  // slider text height
//--------------------------------------------------------
void prepareResolutionValues()
{
  xs = round(width*0.65);
  xt = xs*102/100;
  bh = height / 10;
  yh = (height - bh) / 14;
  sh = yh-2;
  sw = width - xs;
  th = sh*7/10;
  viewDistance = (700 - height) / -2;   //???
  //println("dist="+viewDistance);
}
//--------------------------------------------------------
void setup() 
{
  //orientation (LANDSCAPE);     // for Android
  //size (displayWidth, displayHeight, P3D);
//  size(1280, 768, P3D);     
//  size(800, 600, P3D);     
  size(1112, 834, P3D);     // change height to 834 before saving image
//  size(480, 320, P3D);       // LG 610        
  println(title);
  prepareResolutionValues();
  
  scrollbar[0] = new HScrollbar(xs, 12,       sw, sh, 0, 360, 160);  // ambient.color
  scrollbar[1] = new HScrollbar(xs, 12+yh*1,  sw, sh, 0, 255,  55);  // ambient.intensity
  scrollbar[2] = new HScrollbar(xs, 12+yh*2,  sw, sh, 0, 360,  50);  // emissive.color
  scrollbar[3] = new HScrollbar(xs, 12+yh*3,  sw, sh, 0, 255,  50);  // emissive.intensity
  scrollbar[4] = new HScrollbar(xs, 12+yh*4,  sw, sh, 0, 255, 120);  // specular
  scrollbar[5] = new HScrollbar(xs, 12+yh*5,  sw, sh, 0, 360,  40);  // specular.color
  scrollbar[6] = new HScrollbar(xs, 12+yh*6,  sw, sh, 0, 255, 130);  // specular.intensity
  scrollbar[7] = new HScrollbar(xs, 12+yh*7,  sw, sh, 0, 100,  55);  // shininess
  scrollbar[8] = new HScrollbar(xs, 12+yh*8,  sw, sh, 0, 360, 200);  // directed.color
  scrollbar[9] = new HScrollbar(xs, 12+yh*9,  sw, sh, 0, 255,   0);  // directed.intensity
  scrollbar[10]= new HScrollbar(xs, 12+yh*10, sw, sh, 0, 360,  60);  // pointlight.color
  scrollbar[11]= new HScrollbar(xs, 12+yh*11, sw, sh, 0, 255, 100);  // pointlight.intensity
  scrollbar[12]= new HScrollbar(xs, 12+yh*12, sw, sh, 0, 360,  32);  // spotlight.color
  scrollbar[13]= new HScrollbar(xs, 12+yh*13, sw, sh, 0, 255,   0);  // spotlight.intensity
 
  int bh = 2*sh;
  int buttonY = height -bh -1;
  int bp1 = xs,         bw1 = sw * 5/20;
  int bp2 = bp1+bw1+1,  bw2 = sw * 8/20;
  int bp3 = bp2+bw2+1,  bw3 = sw * 3/20;
  int bp4 = bp3+bw3+1,  bw4 = sw * 4/20;
  ResetButton  = new Button(bp1, buttonY, bw1, bh, "reset");
  RandomButton = new Button(bp2, buttonY, bw2, bh, "random");
  OffButton    = new Button(bp3, buttonY, bw3, bh, "off");
  SaveButton   = new Button(bp4, buttonY, bw4, bh, "save");

  initLights();
}
//--------------------------------------------------------
void draw() 
{
//  if (doSmoothing) smooth(8);  // use only with processing >= v2.0
//  else noSmooth();
  
  background (20+iAmbient / 2);
	fill (255);
	rect(width / 1.5, 0, width, height);
  
  //ortho();     // not working with JavaScript 
  noLights();
  hint (DISABLE_DEPTH_TEST);    

  handleScrollbars();

  // draw buttons
  strokeWeight(0);
  ResetButton.update();
  RandomButton.update();
  OffButton.update();
  SaveButton.update();

  hint (ENABLE_DEPTH_TEST);   // 3D rendering
  translate(xs/2, height/2, viewDistance);
  setLights();
  drawScene();
  
  if (mousePressed && (mouseX < xs-20))
    rotx = (height/2 - mouseY) * 0.004;  
}
//----------------------------------------------------------
void tooglePause()
{
  isPaused = !isPaused;
  if (isPaused) noLoop();  // do not waste cpu time for pausing! 
  else loop();
}
//--------------------------------------------------------
void mousePressed() 
{
//  println("mousePressed: "+isPaused);  // funktioniert nicht ???
//  if (isPaused) tooglePause();
}
//--------------------------------------------------------
void mouseReleased() 
{ 
  if (ResetButton.mouseIsOver()) resetValues();
  if (RandomButton.mouseIsOver()) setRandomValues();
  if (OffButton.mouseIsOver()) turnOffIntensityValues();
  if (SaveButton.mouseIsOver()) save("LightningTest.jpg");
}
//--------------------------------------------------------
void keyPressed()
{
  if (debugMode)
  println (" keyPressed: " + keyCode  + " '" + key + "'  " + int(key)+ "    coded="+(key == CODED));
  if (key == CODED)
  switch (keyCode)
  {        case    4: println ("exit");  // Android Exit button
    break; case   82: println ("menu");  // Android Menu button  
  } 
  else switch (key)
  {        case ' ':  doRotation = !doRotation;
    break; case '+':  viewDistance++;
    break; case '-':  viewDistance--;
    break; case 'a':  drawAxis = !drawAxis;
    break; case 'd':  frameRate(200);          // debug speed 
    break; case 'f':  showFPS = !showFPS;
    break; case 'o':  doSmoothing = !doSmoothing;
    break; case 'w':  showResolution = !showResolution;
    break; case 'i':  turnOffIntensityValues();
    break; case 'n':  setRandomValues();
    break; case 'p':  tooglePause(); 
    break; case 'r':  resetValues();
    break; case 's':  save("LightningTest.jpg");
  }  
}
//--------------------------------------------------------
void showText (String infoText, int x, int y)
{
  //println (infoText + "   "+round(x) + "    "+y);
  fill(44);
  text(infoText, x+1, y+1);
  fill(textColor);
  text(infoText, x, y);
}
//--------------------------------------------------------
void resetValues()     // set original values
{
  for (int ni=0; ni<14; ni++)
    scrollbar[ni].resetValue();
}
//--------------------------------------------------------
void setRandomValues()
{
  for (int ni=0; ni<14; ni++)
    scrollbar[ni].randomize();
  scrollbar[1].setValue(scrollbar[1].value() * 0.3);  // ambient.intensity
  scrollbar[3].setValue(scrollbar[3].value() * 0.2);  // emissive.intensity
}
//--------------------------------------------------------
void turnOffIntensityValues()
{
  scrollbar[1].setValue(0);
  scrollbar[3].setValue(0);
  scrollbar[6].setValue(0);
  scrollbar[9].setValue(0);
  scrollbar[11].setValue(0);
  scrollbar[13].setValue(0);
}
//--------------------------------------------------------
int YT(int index)
{
  return scrollbar[index].centerY();
}
//--------------------------------------------------------
void handleScrollbars()
{  
  textSize(th);
  specular(0, 0, 0);   

  // handle scrollbar
  strokeWeight(0);
  iAmbientHue  = int(scrollbar[0].update());  
  iAmbient     = int(scrollbar[1].update());
  iEmissiveHue = int(scrollbar[2].update());
  iEmissiveSat = int(scrollbar[3].update());
  iSpecular    = int(scrollbar[4].update());
  iSpecularHue = int(scrollbar[5].update());
  iSpecularSat = int(scrollbar[6].update());
  fShininess   =     scrollbar[7].update();
  iDirectHue   = int(scrollbar[8].update());
  iDirectSat   = int(scrollbar[9].update());
  iPointHue    = int(scrollbar[10].update());
  iPointSat    = int(scrollbar[11].update());
  iSpotHue     = int(scrollbar[12].update());
  iSpotSat     = int(scrollbar[13].update());

  // not supported from processing v2.0
  // but important speedup using v1.5.1 !!!
  //textMode(SCREEN);  

  textAlign(LEFT, CENTER);
      
  textColor = color(166, 166, 166);                               //-- german --
  showText("ambient.color="       + iAmbientHue, xt, YT(0));  // Umgebungslicht Farbe
  showText("ambient.intensity="      + iAmbient, xt, YT(1));  // Umgebungslicht Intensity
  
  textColor = color(0, 155, 155);
  showText("emissive.color="     + iEmissiveHue, xt, YT(2));  // Strahlung Farbe
  showText("emissive.intensity=" + iEmissiveSat, xt, YT(3));  // Strahlung Intensität

  textColor = color(188, 188, 11);
  showText("specular="              + iSpecular, xt, YT(4));  // Spiegelung
  showText("specular.color="     + iSpecularHue, xt, YT(5));  // Spiegelungsfarbe
  showText("specular.intensity=" + iSpecularSat, xt, YT(6));  // Spiegelungsintensität
  showText("shininess="  + nf(fShininess, 0, 1), xt, YT(7));  // Glanzpunktgröße

  textColor = color(111, 0, 111);
  showText("directed.color="       + iDirectHue, xt, YT(8));  // gerichtetes Licht
  showText("directed.intensity="   + iDirectSat, xt, YT(9));

  textColor = color(55, 55, 155);
  showText("pointlight.color="      + iPointHue, xt, YT(10));  // Punktlicht
  showText("pointlight.intensity="  + iPointSat, xt, YT(11));

  textColor = color(222, 133, 22);
  showText("spotlight.color="        + iSpotHue, xt, YT(12));  // Scheinwerfer 
  showText("spotlight.intensity="    + iSpotSat, xt, YT(13));

  if (showFPS)
  {
    textColor = color(166);
    showText(round(frameRate) + " fps", 10,22);
  }
  if (showResolution)
  {
    textColor = color(166);
    showText(width + "*" + height, 10,22+yh);
  }
  colorMode(HSB, 360,255,255);
  // change scrollbar background color depending of values
  scrollbar[0].barColor = color(iAmbientHue, 255, 255);
  scrollbar[1].barColor = color(127 + iAmbient/2);
  scrollbar[2].barColor = color(iEmissiveHue, 255, 255);
  scrollbar[3].barColor = color(127 + iEmissiveSat/2);
  scrollbar[4].barColor = color(127 + iSpecular/2);
  scrollbar[5].barColor = color(iSpecularHue, 255, 255);
  scrollbar[6].barColor = color(127 + iSpecularSat/2);
  scrollbar[7].barColor = color(127 + fShininess*255/200);
  scrollbar[8].barColor = color(iDirectHue, 255, 255);
  scrollbar[9].barColor = color(127 + iDirectSat/2);
  scrollbar[10].barColor = color(iPointHue, 255, 255);
  scrollbar[11].barColor = color(127 + iPointSat/2);
  scrollbar[12].barColor = color(iSpotHue, 255, 255);
  scrollbar[13].barColor = color(127 + iSpotSat/2);

  initLights();
}
//--------------------------------------------------------
void drawScene()
{
  pushMatrix();
    rotateX(rotx);
    if (doRotation) spin += 0.01;
    rotateY(PI/5 + spin);
    fill(255);
    pushMatrix();
      //strokeWeight(1);
      stroke(127);
      translate(0, 75, 0);
      box(150);
      translate(0, 91, 0);
      box(180, 30, 180);
    popMatrix();
    strokeWeight(0);
    pushMatrix();
      translate(0, -70, 0);
//      sphereDetail(72, 36);
      sphereDetail(36);
      sphere(90);
    popMatrix();
    pushMatrix();
      sphereDetail(18);
      translate(75, 0, 75);  
      sphere(22);
      translate(-150, 0, 0);  
      sphere(22);
      translate(0, 0, -150);  
      sphere(22);
      translate(150, 0, 0);  
      sphere(22);
    popMatrix();
  popMatrix();

  strokeWeight(3);
  
  // directionalLight specular
  pushMatrix();
    stroke(cSpecular);
    line(0, 0, 0, -specularX, -specularY, -specularZ);
  popMatrix();

  // directionalLight
  pushMatrix();
    stroke(cDirectional);
    line(0, 0, 0, -directX, -directY, -directZ);
  popMatrix();

  // pointLight
  if (iPointSat > 0)
  {
    pushMatrix();
      translate(pointX, pointY, pointZ); 
      noStroke();
      fill(cPoint);
      sphereDetail(18);
      sphere(6);
    popMatrix();
  }

  // spotLight
  if (iSpotSat > 0)
  {
    pushMatrix();
      translate(spotPosX, spotPosY, spotPosZ); 
      fill(cSpot);
      noStroke();
      sphere(6);
      stroke(cSpot);
      line(0, 0, 0, spotDirX, spotDirY, spotDirZ);
    popMatrix();
  }

  // axis
  if (drawAxis)
  {
    strokeWeight(1);
    stroke(0,255,0);    line(0, 0, 0, 250, 0, 0);
    stroke(0,0,255);    line(0, 0, 0, 0, 250, 0);
    stroke(255,0,0);    line(0, 0, 0, 0, 0, 250);
  }
}